import csv
import json
import numpy as np
import pandas as pd
import nltk
import spacy
import gensim
from nltk.stem.porter import PorterStemmer
from nltk.stem import WordNetLemmatizer

# Objective is to place all texts (data) in a CSV file, pre-processed.
# The result is to have a file with all texts (data) pre-processed.

# Configuring libraries
np.random.seed(2018)
nltk.download("wordnet")
nlp = spacy.load("pt_core_news_sm")
stemmer = PorterStemmer()
nlp = spacy.load("pt_core_news_sm")
nlp.Defaults.stop_words |= {
    "tudo",
    "coisa",
    "toda",
    "tava",
    "pessoal",
    "dessa",
    "resolvido",
    "aqui",
    "gente",
    "tá",
    "né",
    "calendário",
    "jpb",
    "agora",
    "voltar",
    "lá",
    "hoje",
    "aí",
    "ainda",
    "então",
    "vai",
    "porque",
    "moradores",
    "fazer",
    "rua",
    "bairro",
    "prefeitura",
    "todo",
    "vamos",
    "problema",
    "fica",
    "ver",
    "tô",
}
stop_words_spacy = nlp.Defaults.stop_words
allowed_postags = ["NOUN", "ADJ", "PRON"]


def verificar_palavra_entidade_loc(palavra, entidades_loc):
    """
    Method that check if the word is entity of location.

    Params:
    ----------
    word : String
        - Word.
    entities_loc : List
        - List of location entities recognized by spacy.

    Return:
    ----------
    True: If the word is a location entity.\n
    False: Othwerwise..
    """
    for e in entidades_loc:
        if e.text.lower() == palavra.lower():
            return True

    return False


def lista_para_texto(lista):
    """
    method that transforms a list of words into text.

    Params:
    ----------
    lista : List
        - List of words.

    Return:
    ----------
    texto : String
        - The text containing all the words in the list.
    """
    texto = ""
    for palavra in lista:
        texto += palavra + " "

    return texto.strip()


def lematizacao(palavra):
    """
    Method that performs the lemmatization of a word.

    Paramso:
    ----------
    word : String
        - Word that will surffer from stemming.

    Return:
    ----------
    word : String
        - Word lemmatization.
    """
    return stemmer.stem(WordNetLemmatizer().lemmatize(palavra, pos="v"))


def pre_processamento(texto, titulo):
    """
    Method that performs the pre-processing of a text:
        - Remove stop words.
        - Remove words that are of location entities.
        - Put all text in lower case.
        - Performs the lemmatization of the words.
        - Removes words that are not: substantivos, adjetivos
            e pronomes.

    Params:
    ----------
    text : String
        - Text that will undergo pre-processing.

    Return:
    ----------
    doc_out : List
        - List of words that have gone through pre-processing.
    """
    doc_out = []
    doc = nlp(texto)
    entidades_loc = [entidade for entidade in doc.ents if entidade.label_ == "LOC"]
    for token in doc:
        if (
            token.text not in stop_words_spacy
            and len(token.text) > 3
            and token.pos_ in allowed_postags
            and not verificar_palavra_entidade_loc(token.text, entidades_loc)
        ):
            doc_out.append(lematizacao(token.text))

    texto = lista_para_texto(doc_out)
    return texto


def main():
    # PREPARING FILES.

    fields = ["titulo", "texto"]
    f = csv.writer(open("../dados/textos_limpos.csv", "w", encoding="utf-8"))
    f.writerow(fields)

    # Load data.
    dados = csv.DictReader(open("../dados/textos_videos.csv", encoding="utf-8"))
    textos = []
    titulo_textos = []

    for arq in dados:
        textos.append(arq["texto"])
        titulo_textos.append(arq["titulo"])

    for texto, titulo in zip(textos, titulo_textos):
        t = pre_processamento(texto, titulo)
        f.writerow([titulo, t])


# main()



from plpygis import Geometry
import csv
import sys

# Increasing the limit size of the CSV.
maxInt = sys.maxsize
csv.field_size_limit(maxInt)


def convert_geometry_point(geometry):
    """
    Pre-processing function that turns a code into geometry(Point)
    for a list of floats.

    Params:
    ----------
    geometry : String
      - String that represents a geometric coordinate of a point.

    Return:
    ----------
    coord : List
      - List of cordenates.
    """
    g = Geometry(geometry)
    coord = g.geojson["coordinates"]
    coord.reverse()
    return coord


def convert_geometry_polygon(geometry):
    """
    Pre-processing function that turns a code into geometry(polygon)
    for a list of floats.

    Params:
    ----------
    geometry : String
      - String that represents a geometric coordinate of a polygon.

    Return:
    ----------
    coord : List
      - List of cordenates.
    """
    g = Geometry(geometry)
    coord = g.geojson["coordinates"]
    saida = []
    for c in coord[0]:
        c.reverse()
        saida.append(c)
    return saida


def convert_feature(geometry):
    """
    Pre-processing function that turns a code into geometry(Feature)
    for a list of floats.

    Params:
    ----------
    geometry : String
      - String that represents a geometric coordinate of a feature.

    Return:
    ----------
    coord : List
      - List of cordenates.
    """
    g = Geometry(geometry)
    coord = g.geojson["coordinates"][0][0]
    saida = []
    for c in coord:
        c.reverse()
        saida.append(c)
    return saida


def polygons(localidade="cg"):
    """
    Pre-processing function that writes polygon address data to another
    file with normalized coordinates.

    Params:
    ----------
    localidade : String
      - String representing the city/state("cg", "jp" ou "pb").
    """
    if localidade == "cg":
        arq = csv.DictReader(
            open("./gazetteer/features_campina_ln.csv", "r", encoding="utf-8")
        )
    elif localidade == "jp":
        arq = csv.DictReader(
            open("./gazetteer/features_jp_ln.csv", "r", encoding="utf-8")
        )
    elif localidade == "pb":
        arq = csv.DictReader(
            open("./gazetteer/features_paraiba_ln.csv", "r", encoding="utf-8")
        )

    fields = ["osm_id", "fclass", "name", "type", "coordenates"]
    f = csv.writer(
        open("./gazetteer/processados/" + localidade + "_ln.csv", "w", encoding="utf-8")
    )
    f.writerow(fields)

    for p in arq:
        coord = convert_geometry_polygon(p["geometry"])
        t = [
            p["osm_id"].__str__(),
            p["fclass"].__str__(),
            p["name"].__str__(),
            p["type"].__str__(),
            coord,
        ]
        f.writerow(t)


def points(localidade="cg"):
    """
    Pre-processing function that writes point address data to another
    file with normalized coordinates.

    Params:
    ----------
    localidade : String
      - String representing the city/state("cg", "jp" ou "pb").
    """
    if localidade == "cg":
        arq = csv.DictReader(
            open("./gazetteer/features_campina_pt.csv", "r", encoding="utf-8")
        )
    elif localidade == "jp":
        arq = csv.DictReader(
            open("./gazetteer/features_jp_pt.csv", "r", encoding="utf-8")
        )
    elif localidade == "pb":
        arq = csv.DictReader(
            open("./gazetteer/features_paraiba_pt.csv", "r", encoding="utf-8")
        )

    fields = ["osm_id", "fclass", "name", "type", "coordenates"]
    f = csv.writer(
        open("./gazetteer/processados/" + localidade + "_pt.csv", "w", encoding="utf-8")
    )
    f.writerow(fields)

    for p in arq:
        coord = convert_geometry_point(p["geometry"])
        t = [
            p["osm_id"].__str__(),
            p["fclass"].__str__(),
            p["name"].__str__(),
            p["type"].__str__(),
            coord,
        ]
        f.writerow(t)


def features(localidade="cg"):
    """
    Pre-processing function that writes feature address data to another
    file with normalized coordinates.

    Params:
    ----------
    localidade : String
      - String representing the city/state("cg", "jp" ou "pb").
    """
    if localidade == "cg":
        arq = csv.DictReader(open("./gazetteer/features_campina.csv", mode="r"))
    elif localidade == "jp":
        arq = csv.DictReader(open("./gazetteer/features_jp.csv", mode="r"))
    elif localidade == "pb":
        arq = csv.DictReader(open("./gazetteer/features_paraiba.csv", mode="r"))

    fields = ["osm_id", "fclass", "name", "type", "coordenates"]
    f = csv.writer(
        open("./gazetteer/processados/" + localidade + ".csv", "w", encoding="utf-8")
    )
    f.writerow(fields)

    for p in arq:
        coord = convert_feature(p["geometry"])
        t = [
            p["osm_id"].__str__(),
            p["fclass"].__str__(),
            p["name"].__str__(),
            p["type"].__str__(),
            coord,
        ]
        f.writerow(t)


def main():
    locs = ["cg", "jp", "pb"]

    for loc in locs:
        points(localidade=loc)

    for loc in locs:
        polygons(localidade=loc)

    for loc in locs:
        features(localidade=loc)


# main()



import pytest

from pyelit import Geoparsing


@pytest.fixture
def init_geoparsing():
    geoparsing = Geoparsing()
    return geoparsing


class TestGeoparsing:
    correct_phrase = "eu moro na Rua João Sérgio de Almeida"
    geoparsing = Geoparsing()

    def test_geoparsing_simple(self, init_geoparsing):
        result = init_geoparsing.geoparsing(text=self.correct_phrase, case_correct=True)

        address_correct = (
            "Rua João Sérgio de Almeida, "
            + "Malvinas, Campina Grande, Paraíba, 58433-395"
        )
        assert type(result) == list
        assert result[0]["address"] == address_correct

    def test_geoparsing_case_using_gazetteer(self, init_geoparsing):
        result = init_geoparsing.geoparsing(text=self.correct_phrase, gazetteer_cg=True)

        address_correct = (
            "Rua João Sérgio de Almeida 1015-1015, "
            + "Bodocongó, Campina Grande, Paraíba, 58433-395"
        )
        assert type(result) == list
        assert result[0]["address"] == address_correct
        assert result[0]["city"] == "Campina Grande"
        assert result[0]["occurrences_in_text"] == 1

    def test_geoparsing_simple_exception(self, init_geoparsing):
        with pytest.raises(Exception):
            init_geoparsing.geoparsing(text="Eu moro na rua joão sérgio")

    def test_geoparsing_case_correct_exception(self, init_geoparsing):
        with pytest.raises(Exception):
            init_geoparsing.geoparsing(
                text="Eu moro na rua joão sérgio de almeida", case_correct=True
            )

    def test_geoparsing_case_using_gazetteer_exception(self, init_geoparsing):
        with pytest.raises(Exception):
            init_geoparsing.geoparsing(
                text="Eu moro na rua joão sérgio", gazetteer_cg=True
            )



import os
import csv
import unicodedata
from googletrans.models import Translated

import truecase
from googletrans import Translator

import spacy
import geocoder

from nltk.stem.porter import PorterStemmer, re

from .exceptions.GeoparsingException import GeoparsingException
from .utils.utils import string_to_list


class Geoparsing:
    """
    Class responsible for performing Geoparsing.
    """

    def __init__(self):
        """
        Class constructor. Where all attributes are started
        and undergo pre-processing.
        """
        self.translator = Translator()
        self.stemmer = PorterStemmer()
        self.nlp = spacy.load("pt_core_news_sm")
        self.nlp.Defaults.stop_words |= {
            "vamos",
            "olha",
            "pois",
            "tudo",
            "coisa",
            "toda",
            "tava",
            "pessoal",
            "dessa",
            "resolvido",
            "aqui",
            "gente",
            "tá",
            "né",
            "calendário",
            "jpb",
            "agora",
            "voltar",
            "lá",
            "hoje",
            "aí",
            "ainda",
            "então",
            "vai",
            "porque",
            "moradores",
            "fazer",
            "prefeitura",
            "todo",
            "problema",
            "fica",
            "ver",
            "tô",
        }
        self.stop_words_spacy = self.nlp.Defaults.stop_words
        self.residential = {}
        self.gazetteer = {}
        ROOT = os.path.abspath(os.path.dirname(__file__))
        fname = ROOT + "/gazetteer/processados"
        self.gazetteer_ln = csv.DictReader(
            open(fname + "/gazetteerpb.csv", "r", encoding="utf-8")
        )
        self.pre_process(self.gazetteer_ln)

    def pre_process(self, gazetteer):
        """
        Method that performs the pre-processing of the gazetteer.
        Loads the information fron the gazetteer into a dictionary.

        Params:
        ----------
        gazetteer: dict
            - Python native library object: `CSV`.
        """
        for row in gazetteer:
            self.gazetteer[self.remove_accents(row["osm_id"])] = (
                row["coordenates"],
                row["fclass"],
                self.remove_accents(row["name"].lower()),
                row["type"],
            )

    def remove_accents(self, input_str):
        """
        Method that removes accents from words.

        Params:
        ----------
        input_str: String
            - Input to be removed the accents
        """
        nfkd_form = unicodedata.normalize("NFKD", input_str)
        only_ascii = nfkd_form.encode("ASCII", "ignore")
        return only_ascii.decode("utf-8")

    def concantenate_address(self, list_address, exclude=False):
        """
        Method that concatenates addresses.

        Params:
        ----------
        list_address: List
            - List containing all addresses found.

        Return:
        ----------
        out : List
            - List of addresses concatenates.
        """
        if exclude:
            out = []
        else:
            out = [e for e in list_address]
        for i in range(len(list_address) - 1):
            for j in range(i + 1, len(list_address)):
                temp = str(list_address[i]) + " " + str(list_address[j])
                out.append(temp)
        return out

    def check_reliability_address(self, address):
        """
        Method that check if an address is from Paraíba
        and whether its reliability is greater than or equal to 5.


        Params:
        ----------
        address: Dict
            - Dictionary containing all address information.

        Return:
        ----------
        True: If the address meets the requirements.
        False: otherwise
        """
        if address["confidence"] >= 5:
            # ", campina grande" in end['address'].lower() and
            # if (", paraíba" in end['address'].lower()):
            return True
        else:
            return False

    def check_address(self, location_entities, limit):
        """
        Method that checks if the addresses are correct.
            - Find the locations of the location entities.
            - Check if is from PB and your reliability
              (`check_reliability_address`).
            - Concatenates of addresses.
            - Sort addresses by reliability.

        Params:
        ----------
        location_entities : List
             - List of locations entities.
        limit : Integer
            - Number of addresses you want to return.

        Return:
        ----------
        out : Tuple
            - One tuple of type (Boolean - List).
            If you have found at least one address in these requirements.
            List of addresses(len = limit))
        """
        addresses = []
        for location in location_entities:
            location = str(location)
            g = geocoder.arcgis(location)
            address = g.json
            if address is not None:
                addresses.append(address)

        correct_addresses = []
        for addr in addresses:
            if self.check_reliability_address(addr):
                correct_addresses.append(addr)

        if len(correct_addresses):
            addr_final = correct_addresses[0]
            for addr in correct_addresses:
                if addr["confidence"] > addr_final["confidence"]:
                    addr_final = addr
            addrs_ = sorted(correct_addresses, key=lambda end: end["confidence"])
            return (True, addrs_[0:limit])
        else:
            return (False, [])

    def search_next_index(self, list_best_address):
        """
        Method that search a new position to be added
        in list of best addresses


        Params:
        ----------
        list_best_address : List
            - List containing all addresses found.

        Return:
        ----------
        out : Integer
            - Position where the new address must be added.
        """
        for index in range(len(list_best_address)):
            if list_best_address[index]["type_class"] == "geral":
                return index

        return len(list_best_address) - 1

    def insert_ordened_to_priority(self, result, address, type_):
        """
        Methodo that insert in list of best addresses order by priority.

        Params:
        ----------
        result : List
            - List of best addresses.
        address : Dict
            - Address to be insert in list.
        type_ : String
            - Type of address.
        """
        if address not in result:
            if type_ == "school":
                address["type_class"] = "school"
                result.insert(0, address)
            elif type_ == "residential":
                address["type_class"] = "residential"
                result.insert(self.search_next_index(result), address)
            else:
                address["type_class"] = "geral"
                result.append(address)

    def choose_best_addresses(self, adresses, text, addresses_concatenated, cities):
        """
        Method that performs the chose of the best addresses found.

        Algorithms implemented:
            - Sort by priority levels.
            - Filter by addresses that are in a certain neighborhood that is
            also in these filtered addresses.
            - Addresses that are more repeated in the text.
            - Addresses that are StreatName.
            - Addresses that are contained in the cities found in the text.

        Params:
        ----------
        adresses : Dict
            - Adddresses dictionary and its cordinates .
        text : String
            - Text that is going through geoparsing..
        addresses_concatenated : List
            - List with all addresses concatenated   with each other.
        cities : List
            - List of city names found.

        Return:
        ----------
        result : List
            - List of best addresses objects.
        """
        result = []
        for location in adresses.keys():
            coord, type_ = adresses[location]
            lat, lon = string_to_list(coord)
            location_ = str(lat[0]) + ", " + str(lon[0])
            addr = geocoder.reverse(location=location_, provider="arcgis")
            addr = addr.json
            if addr is not None:
                addr["occurrences_in_text"] = text.count(location)
                result.append(addr)
                self.insert_ordened_to_priority(result, addr, type_)

        result = sorted(result, key=lambda addr: addr["occurrences_in_text"])

        new_result = []
        for index in range(len(result) - 1, -1, -1):
            location = result[index]
            if location["raw"].__contains__("address"):
                location_district = location["raw"]["address"]["District"].lower()
                if location_district in adresses.keys():
                    new_result.insert(0, location)
                else:
                    new_result.append(location)
            else:
                if location["raw"]["name"].lower() in adresses.keys():
                    new_result.insert(0, location)
                else:
                    new_result.append(location)
        result = new_result

        for location in addresses_concatenated:
            location_ = str(location)
            addr = geocoder.arcgis(location_)
            addr = addr.json
            result.insert(0, addr)

        new_result = []
        for index in range(len(result) - 1, -1, -1):
            if result[index].__contains__("quality"):
                if result[index]["quality"] == "StreetName":
                    new_result.insert(0, result[index])
                else:
                    new_result.append(result[index])
            else:
                new_result.append(result[index])

        result = new_result

        new_result = []
        if cities != []:
            for index in range(len(result) - 1, -1, -1):
                for city in cities:
                    if result[index].__contains__("quality"):
                        if city in result[index]["address"].lower():
                            new_result.insert(0, result[index])
                    else:
                        loc_city = str(result[index]["raw"]["address"]["City"]).lower()
                        if loc_city == city:
                            new_result.insert(0, result[index])
            result = new_result

        return result

    def filter_address_text(self, text):
        """
        Method that performs the filtering of the text addresses
        that are in the gazetteer.

        Params:
        ----------
        text : String
            - Text that to performs the geoparsing..

        Return:
        ----------
        result : Dict
            - Addresses dictionary and its cordinates.
        """
        general_addresses = {}

        text = self.remove_accents(text)

        for osm_id in self.gazetteer.keys():
            address = self.gazetteer.get(osm_id)[2]
            auxiliary_address = address.split()
            if auxiliary_address[0] == "rua":
                auxiliary_address = auxiliary_address[1:]
            if len(auxiliary_address) > 1 or self.gazetteer[osm_id][1] == "suburb":
                address = address.replace("(", "")
                address = address.replace(")", "")
                txt_contains_addr = re.search("\\b" + address + "\\b", text)
                if txt_contains_addr:
                    addr_repeated = self.repeated_address(
                        general_addresses.keys(), address
                    )
                    if not addr_repeated:
                        general_addresses[address] = (
                            self.gazetteer[osm_id][0],
                            self.gazetteer[osm_id][1],
                        )

        cities = [
            str(addr)
            for addr in general_addresses.keys()
            if general_addresses[addr][1] == "city"
        ]

        addresses_concatenated = [str(addr) for addr in general_addresses.keys()]
        addresses_concatenated = self.concantenate_address(
            addresses_concatenated, exclude=True
        )
        result = self.choose_best_addresses(
            general_addresses, text, addresses_concatenated, cities
        )
        return result

    def repeated_address(self, addresses, address):
        """
        Method that checks whether an address is repeated.

        Params:
        ----------
        addresses: List
            - List of addresses
        address: String
            - Address

        Return:
        ----------
        True: if the address is repeated.
        False: Otherwise.
        """
        for addr in addresses:
            if address in addr:
                return True
        return False

    def geoparsing(self, text, case_correct=False, limit=5, gazetteer_cg=False):
        """
        Method that performs the geoparsing of text,

        NOTE: use the geoparsing without the correct case and withour
        the gazetteer will give you poor results.

        Params:
        ----------
        text : String
            - Text that to performs the geoparsing.
        case_correct: Bool
            - If the text is with correct case.
        limit: Int
            - Maximum limit of returned addresses.
        gazetteer_cg: Bool
            - If you want to use the gazetteer with locations in
            the state of Paraíba.

        Return:
        ----------
        result : List
            - List of addresses.
        """
        if gazetteer_cg:
            result = self.filter_address_text(text.lower())
            if result:
                return result
            else:
                raise GeoparsingException()
        else:
            if case_correct:
                doc = self.nlp(text)
                ents_loc = list(
                    filter(
                        lambda entity: entity.label_ == "LOC" or entity.label_ == "GPE",
                        doc.ents,
                    )
                )
                address_found = self.concantenate_address(ents_loc)
                result = self.check_address(address_found, limit)
                if result[0]:
                    return result[1]
                else:
                    raise GeoparsingException()
            else:
                text = truecase.get_true_case(text)

                text_en = self.translator.translate(text, dest="en")
                text_en = text_en.text
                text_true_case = truecase.get_true_case(text_en)

                text_pt = self.translator.translate(text_true_case, src="en", dest="pt")
                text = text_pt.text

                return self.geoparsing(text, case_correct=True)



import pytest

from pyelit import TopicModeling


@pytest.fixture
def init_topic_modeling():
    tModeling = TopicModeling()
    return tModeling


class TestTopicModeling:
    tModeling = TopicModeling()

    def test_topic_modeling_simple(self, init_topic_modeling):
        result = init_topic_modeling.rate_text(
            "Tem um buraco na minha rua e a prefeitura não faz nada"
        )

        assert result[0][0] == 0  # Saneamento

    def test_topic_modeling_print_keywords(self, init_topic_modeling):
        print(init_topic_modeling.print_keywords())
        assert init_topic_modeling.print_keywords()[0] == (
            0,
            '0.016*"água" + 0.015*"esgoto" + 0.010*"calçamento" + '
            + '0.010*"casa" + 0.009*"buraco"',
        )

    def test_topic_modeling_print_topics(self, init_topic_modeling):
        assert init_topic_modeling.print_topics() == {
            0: "Sanitation",
            1: "Traffic",
            2: "Construction",
            3: "Several",
        }

    def test_topic_modeling_get_topic(self, init_topic_modeling):
        assert init_topic_modeling.get_topic(0) == "Sanitation"
        assert init_topic_modeling.get_topic(1) == "Traffic"
        assert init_topic_modeling.get_topic(2) == "Construction"
        assert init_topic_modeling.get_topic(3) == "Several"

    def test_topic_modeling_represent_topics(self, init_topic_modeling):
        ids = [0, 1, 2, 3]
        names = ["SANEAMENTO", "TRÂNSITO", "OBRAS", "DIVERSOS"]
        init_topic_modeling.represent_topics(ids, names)

        assert init_topic_modeling.get_topic(0) == "SANEAMENTO"
        assert init_topic_modeling.get_topic(1) == "TRÂNSITO"
        assert init_topic_modeling.get_topic(2) == "OBRAS"
        assert init_topic_modeling.get_topic(3) == "DIVERSOS"



import os
import nltk
import random
import spacy
import gensim
import numpy as np
import pandas as pd
from gensim import corpora, models
from gensim.models.coherencemodel import CoherenceModel
import matplotlib.pyplot as plt
from gensim.test.utils import datapath
from sklearn.model_selection import train_test_split


def main_cross_val():
    # Useing cross-validation
    # Configuring libraries for better results.
    np.random.seed(2018)
    nltk.download("wordnet")
    nlp = spacy.load("pt_core_news_sm")

    # PREPARING FILES.
    data = pd.read_csv("../../dados/textos_limpos.csv")
    data.drop_duplicates(["texto"], inplace=True)
    texts = data["texto"]
    texts = [str(texto) for text in texts]
    cross = kfoldcv(texts)
    print("Total: ", len(texts))

    for train_tests in cross:
        train = train_tests[0]
        test = train_tests[1]

        processed_docs = [t.split() for t in train]
        processed_test = [t.split() for t in test]

        print("Train: ", len(processed_docs))
        print("Tests: ", len(processed_test))

        # Creating dictionary of words.
        dictionary = gensim.corpora.Dictionary(processed_docs)

        # Gensim Filter Extremes
        # Filter tokens that appear in less than 15 documents.
        # or in more than 0.5 documents (fraction of the total corpus size).
        # After these two steps, keep only 100000.
        dictionary.filter_extremes(no_below=15, no_above=0.5, keep_n=100000)

        # Bag of Words.
        bow_corpus = [dictionary.doc2bow(doc) for doc in processed_docs]

        # Using TF-IDF.
        tfidf = models.TfidfModel(bow_corpus)
        corpus_tfidf = tfidf[bow_corpus]

        # Creating and training the model.
        lda_model_tfidf = gensim.models.LdaMulticore(
            corpus_tfidf,
            num_topics=4,
            id2word=dictionary,
            passes=10,
            workers=4,
            alpha=0.01,
            eta=0.9,
        )

        coherence_model(lda_model_tfidf, processed_test, corpus_tfidf, dictionary)


def coherence_model(lda_model_, tests, corpus, dictionary):
    coherence_model_lda = CoherenceModel(
        model=lda_model_,
        texts=tests,
        corpus=corpus,
        dictionary=dictionary,
        coherence="c_v",
    )
    coherence_lda = coherence_model_lda.get_coherence()
    print("\nCoherence Score LDAModelTfIdf: ", coherence_lda)


def kfoldcv(dados, k=6, seed=42):
    size = len(dados)
    subset_size = round(size / k)
    random.Random(seed).shuffle(dados)
    subsets = [dados[x : x + subset_size] for x in range(0, len(dados), subset_size)]
    kfolds = []
    for i in range(k):
        test = subsets[i]
        train = []
        for subset in subsets:
            if subset != test:
                for ss in subset:
                    train.append(ss)
        kfolds.append((train, test))

    return kfolds


def main():
    # Configuring libraries for better results.
    np.random.seed(2018)
    nltk.download("wordnet")
    nlp = spacy.load("pt_core_news_sm")

    # PREPARING FILES.
    data = pd.read_csv("../../dados/textos_limpos.csv")
    data.drop_duplicates(["texto"], inplace=True)

    processed_docs = data["texto"].map(lambda text: text.split())
    print(processed_docs[:10])

    # Creating dictionary of words.
    dictionary = gensim.corpora.Dictionary(processed_docs)

    # Gensim Filter Extremes
    # Filter tokens that appear in less than 15 documents
    # or in more than 0.5 documents (fraction of the total corpus size).
    # After these two steps, keep only 100000.
    dictionary.filter_extremes(no_below=15, no_above=0.5, keep_n=100000)

    # Bag of Words.
    bow_corpus = [dictionary.doc2bow(doc) for doc in processed_docs]

    # Using TF-IDF.
    tfidf = models.TfidfModel(bow_corpus)
    corpus_tfidf = tfidf[bow_corpus]

    # Creating and training the model.
    # lda_model = gensim.models.LdaMulticore(corpus_tfidf, num_topics=4,
    #                                        id2word=dictionary, passes=10,
    #                                        workers=4, alpha=0.01, eta=0.9)
    # slda_model.save("./modelo/meu_lda_model")

    ROOT = os.path.abspath(os.path.dirname(__file__))
    fname = datapath(ROOT + "/modelo/meu_lda_model")
    model = gensim.models.LdaMulticore.load(fname=fname)

    def coherence_model(lda_model_, processed_docs, corpus_tfidf, dictionary):
        coherence_model_lda = CoherenceModel(
            model=lda_model_,
            texts=processed_docs,
            corpus=corpus_tfidf,
            dictionary=dictionary,
            coherence="c_v",
        )
        coherence_lda = coherence_model_lda.get_coherence()
        print("\nCoherence Score LDAModelTfIdf load: ", coherence_lda)

    coherence_model(model, processed_docs, corpus_tfidf, dictionary)

    def compute_num_steps(dct, corpus_tfidf, texts, limit, start, step):
        coherence_values = []
        model_list = []
        for passes in range(start, limit, 1):
            model = gensim.models.LdaMulticore(
                corpus_tfidf,
                num_topics=4,
                id2word=dictionary,
                passes=passes,
                workers=4,
                alpha=0.01,
                eta=0.9,
            )
            model_list.append(model)
            coherencemodel = CoherenceModel(
                model=model,
                texts=texts,
                corpus=corpus_tfidf,
                dictionary=dct,
                coherence="c_v",
            )
            coherence_values.append(coherencemodel.get_coherence())

        return model_list, coherence_values

    model_list, coherence_values = compute_num_steps(
        dictionary, corpus_tfidf, processed_docs, 13, 9, 1
    )

    limit = 13
    start = 9
    step = 1
    x = range(start, limit, step)
    plt.plot(x, coherence_values)
    plt.xlabel("Steps")
    plt.ylabel("Coherence score")
    plt.legend(("coherence_values"), loc="best")
    plt.show()

    for m, cv in zip(x, coherence_values):
        print("Step =", m, " has Coherence Value of", round(cv, 4))


main()
# main_cross_val()



import os
import nltk
import spacy
import gensim
import numpy as np
from nltk.stem import WordNetLemmatizer
from nltk.stem.porter import PorterStemmer
from gensim.test.utils import datapath


class TopicModeling:
    """
    Topic Modeling Class with a coherence score of 0.52.
    As an unsupervised learning approach is used, no accuracy tests
    were performed, but those that were performed manually
    obtained good results.

    It allows from a text to be able to describe which topic that
    text belongs to with a good probability.
    """

    def __init__(self):
        """
        Class constructor. Starts the main objects/attributes
        for its operation.
        """
        self.stemmer = PorterStemmer()
        self.nlp = spacy.load("pt_core_news_sm")
        self.nlp.Defaults.stop_words |= {
            "tudo",
            "coisa",
            "toda",
            "tava",
            "pessoal",
            "dessa",
            "resolvido",
            "aqui",
            "gente",
            "tá",
            "né",
            "calendário",
            "jpb",
            "agora",
            "voltar",
            "lá",
            "hoje",
            "aí",
            "ainda",
            "então",
            "vai",
            "porque",
            "moradores",
            "fazer",
            "rua",
            "bairro",
            "prefeitura",
            "todo",
            "vamos",
            "problema",
            "fica",
            "ver",
            "tô",
        }
        self.stop_words_spacy = self.nlp.Defaults.stop_words
        np.random.seed(2018)
        nltk.download("wordnet")
        self.allowed_postags = ["NOUN", "ADJ", "PRON"]
        # Carrega o modelo.
        ROOT = os.path.abspath(os.path.dirname(__file__))
        fname = datapath(ROOT + "/modelo/meu_lda_model")
        self.model = gensim.models.LdaMulticore.load(fname=fname)
        self.topics = {}
        self.represent_topics(
            [0, 1, 2, 3], ["Sanitation", "Traffic", "Construction", "Several"]
        )

    def pre_processing(self, text):
        """
        Method that performs the pre-processing of a text:
            - Remove stop words.
            - Remove words that are of location entities.
            - Put all text in lower case.
            - Performs the lemmatization of the words.
            - Removes words that are not: substantivos, adjetivos
             e pronomes.

        Params:
        ----------
        text : String
            - Text that will undergo pre-processing.

        Return:
        ----------
        doc_out : List
            - List of words that have gone through pre-processing.
        """
        doc_out = []
        doc = self.nlp(text)
        location_entities = [entity for entity in doc.ents if entity.label_ == "LOC"]
        for token in doc:
            if (
                token.text not in self.stop_words_spacy
                and len(token.text) > 3
                and token.pos_ in self.allowed_postags
                and not self.is_entities_loc(token.text, location_entities)
            ):
                doc_out.append(self.lemmatization(token.text))

        return doc_out

    def lemmatization(self, word):
        """
        Method that performs the lemmatization of a word.

        Paramso:
        ----------
        word : String
            - Word that will surffer from stemming.

        Return:
        ----------
        word : String
            - Word lemmatization.
        """
        return self.stemmer.stem(WordNetLemmatizer().lemmatize(word, pos="v"))

    def is_entities_loc(self, word, location_entities):
        """
        Method that check if the word is entity of location.

        Params:
        ----------
        word : String
            - Word.
        location_entities : List
            - List of location entities recognized by spacy.

        Return:
        ----------
        True: If the word is a location entity.\n
        False: Othwerwise..
        """
        for entity in location_entities:
            if entity.text.lower() == word.lower():
                return True

        return False

    def print_keywords(self, max_number_words=None):
        """
        Method that will print the keywords for each of topics in the model.

        Params:
        ----------
        max_number_words: Int
            - Maximum number of words that representing a topic to be returned

        Return:
        ----------
        topics : List
            - List of keywords for topics in the model.
        """
        if max_number_words is None:
            max_number_words = 5
        topics = []
        for topic in self.model.print_topics(-1, max_number_words):
            topics.append(topic)
        return topics

    def print_topics(self):
        """
        Method that will print each of topics in the model.

        Return:
        ----------
        topics : List
            - List of topics in the model.
        """
        return self.topics

    def represent_topics(self, ids_topics, names_topics):
        """
        Method that will set the values to the topics.

        NOTE: The two must come in the same order, name in position 0
        is from id in position 0.


        Params:
        ----------
        ids_topics: List
            - List of ids in the topics.
        names_topics: List
            - List of names of topics.
        """
        for id_topic, name in zip(ids_topics, names_topics):
            self.topics[id_topic] = name

    def get_topic(self, id_topic):
        """
        Method that returns the representation of the topic.

        Params:
        ----------
        id_topic: Int
            - Integer that represent the topic.

        Return:
        ----------
        topics: String
            - The name that represent the topic with the `id_topic`.
        """
        return self.topics[id_topic]

    def rate_text(self, text):
        """
        Method that will return from which topic the text is more
        likely to belong to.

        Params:
        ----------
        text : String
            - Text that will to be evaluate.

        Return:
        ----------
        result : List
            - List of tuples with id of topic that the text belongs
            and probability.
        """
        bow_vector = self.model.id2word.doc2bow(self.pre_processing(text))
        result = self.model.get_document_topics(bow_vector)
        result = sorted(result, reverse=True, key=lambda t: t[1])
        return result



from setuptools import setup, find_packages
from setuptools.command.develop import develop
from setuptools.command.install import install
from subprocess import check_call

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("./requirements.txt", "r") as f:
    requirements = []
    for line in f:
        requirements.append(line.strip())


class PostDevelopCommand(develop):
    """Pos-installation for development mode."""

    def run(self):
        develop.run(self)


class PostInstallCommand(install):
    """Pos-installation for installation mode."""

    def run(self):
        check_call("python3 -m spacy download pt_core_news_sm".split())
        install.run(self)


keywords = "geoparsing topic modeling locations text issues problems urbans"

setup(
    name="PyElit",
    version="0.2.0",
    author="Rich Ramalho",
    author_email="richelton14@gmail.com",
    description="Extraction of Locations and Issues of a Text",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=requirements,
    include_package_data=True,
    url="https://github.com/Rickecr/PyElit",
    project_urls={
        "Código fonte": "https://github.com/Rickecr/PyElit",
    },
    keywords=keywords,
    license="MIT",
    cmdclass={
        "develop": PostDevelopCommand,
        "install": PostInstallCommand,
    },
)
